package cn.myapps.email.controller;

import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import cn.myapps.run.EmailApp;

@RunWith(SpringRunner.class)
@SpringBootTest(classes= EmailApp.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class EmailControllerTest extends AbstractControllerTests {

    /**
     * 邮箱用户登录
     * @throws Exception
     */
    @Test
    public void testEmailLogin() throws Exception {

        String url = "/api/email/userLogin";

        String content = "{\"ownerid\": \"11111\",\"name\": \"专员\",\"account\": \"gary\",\"password\": \"MGbrdQxczKbpdytf\"}";
        mockMvc.perform(MockMvcRequestBuilders.post(url)
                .contentType(MediaType.APPLICATION_JSON)
                .content(content)
                .cookie(cookie))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.errcode").value(0))
                .andDo(print());

        testGetFolderList();
    }

    /**
     * 获取邮箱后缀
     * @throws Exception
     */
    @Test
    public void testGetEmailSuffix() throws Exception {

        String url = "/api/email/getEmailSuffix";

        mockMvc.perform(MockMvcRequestBuilders.get(url)
                .contentType(MediaType.APPLICATION_JSON)
                .cookie(cookie))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.errcode").value(0))
                .andDo(print());

    }

    /**
     * 获取邮箱文件夹中邮件列表
     * @throws Exception
     */
    @Test
    public void testGetEmailsByFolder() throws Exception {

        String url = "/api/email/getFolderEmails?folderId=-877551452";

        mockMvc.perform(MockMvcRequestBuilders.get(url)
                .contentType(MediaType.APPLICATION_JSON)
                .cookie(cookie))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.errcode").value(0))
                .andDo(print());
    }

    /**
     * 获取邮件详情
     * @throws Exception
     */
    @Test
    public void testGetEamil() throws Exception {

        String url = "/api/email/getEmail?folderId=-877551452&emailId=776";

        mockMvc.perform(MockMvcRequestBuilders.get(url)
                .contentType(MediaType.APPLICATION_JSON)
                .cookie(cookie))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.errcode").value(0))
                .andDo(print());
    }

    /**
     * 删除邮件
     * @throws Exception
     */
    @Test
    public void testDeleteEamil() throws Exception {
        String url = "/api/email/removeEmail?folderId=-877551452";
        String content = "[\"928\"]";
        mockMvc.perform(MockMvcRequestBuilders.delete(url)
                .contentType(MediaType.APPLICATION_JSON)
                .content(content)
                .cookie(cookie))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.errcode").value(0))
                .andDo(print());
    }

    /**
     * 发送邮件
     * @throws Exception
     */
    @Test
    public void testSendEmail() throws Exception {
        String url = "/api/email/sendEmail?folderId=1077161872";
        String content = "{\"from\": \"肖专员-华北区(销售专员) <gary@weioa365.com>\", \t\"to\": \"肖专员-华北区(销售专员) <gary@weioa365.com>\", \t\"subject\": \"3333333\", \t\"content\": \"222222\",\t\"attids\": [\"1\"] }";
        mockMvc.perform(MockMvcRequestBuilders.post(url)
                .contentType(MediaType.APPLICATION_JSON)
                .content(content)
                .cookie(cookie))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.errcode").value(0))
                .andDo(print());
    }

    /**
     * 标记邮件
     * @throws Exception
     */
    @Test
    public void testMarkEmail() throws Exception {

        String url = "/api/email/markEmail?folderId=-877551452&mark=0";
        String content = "[\"781\"]";
        mockMvc.perform(MockMvcRequestBuilders.post(url)
                .contentType(MediaType.APPLICATION_JSON)
                .content(content)
                .cookie(cookie))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.errcode").value(0))
                .andDo(print());
    }

    /**
     * 获取邮箱文件夹列表
     * @throws Exception
     */
    @Test
    public void testGetFolderList() throws Exception {

        String url = "/api/email/getFolderList";

        mockMvc.perform(MockMvcRequestBuilders.get(url)
                .contentType(MediaType.APPLICATION_JSON)
                .cookie(cookie))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.errcode").value(0))
                .andDo(print());
    }
}
